#include "demo.h"
#include "etc_driver/driver_ssd1351_font.h"

#define BI_RGB 0L
typedef struct {
    U32 bfSize;
    U32 bfReserved;
    U32 bfOffBits;
    U32 biSize;
    S32 biWidth;
    S32 biHeight;
    U16 biPlanes;
    U16 biBitCount;
    U32 biCompression;
    U32 biSizeImage;
    S32 biXPelsPerMeter;
    S32 biYPelsPerMeter;
    U32 biClrUsed;
    U32 biClrImportant;
} BITMAPFILEHEADER;

typedef struct {
    U8 rgbBlue;
    U8 rgbGreen;
    U8 rgbRed;
    U8 rgbReserved;
} RGBQUAD;
#define EXTRACT_READ32(startaddr, offset) (U32)(startaddr[offset] + (U32)(startaddr[offset + 1] << 8) + (U32)(startaddr[offset + 2] << 16) + (U32)(startaddr[offset + 3] << 24))

// version 0x100
typedef struct {
    unsigned int magic;
    unsigned int version;
    unsigned short w;        // image width
    unsigned short h;        // image height
    unsigned short lpitch;   // allocated memory width 8,16,32,64,128,256,512,1024
    unsigned char bCompress; // 0 : no compress, 1 : RLE
    unsigned char bpp;
} SURF_HEADER;

#define SSD1351_CMD  0 /**< command type */
#define SSD1351_DATA 1 /**< data type */

static void lcd_cmd(U8 cmd)
{
    ssd1351_interface_cmd_data_gpio_write(SSD1351_CMD);
    ssd1351_interface_spi_write(&cmd, 1);
}
static void lcd_data(U8 data)
{
    ssd1351_interface_cmd_data_gpio_write(SSD1351_DATA);
    ssd1351_interface_spi_write(&data, 1);
}
static void lcd_data_len(U8 *buf, int len)
{
    ssd1351_interface_cmd_data_gpio_write(SSD1351_DATA);
    ssd1351_interface_spi_write(buf, len);
}

#define SSD1351_CMD_SET_COLUMN_ADDRESS 0x15 /**< set column address command */
#define SSD1351_CMD_SET_ROW_ADDRESS    0x75 /**< set row address command */
#define SSD1351_CMD_WRITE_RAM          0x5C /**< write ram command */

ALIGN4 static U16 framebuffer[128][128];

/// @brief draw pixle with RGB5656 Color values
/// @param x X coordinate
/// @param y Y coordinate
/// @param clr RGB5656 Color values
__attribute__((always_inline)) void inline fb_draw_pixel(int x, int y, U16 clr)
{
    framebuffer[y][x] = (clr >> 8) | (clr << 8);
}

U16 *fb_get_fremebuffer(int x, int y)
{
    return &framebuffer[y][x];
}

void fb_update_lcd()
{
    lcd_cmd(SSD1351_CMD_SET_COLUMN_ADDRESS);
    lcd_data(0);   // left
    lcd_data(127); // right
    lcd_cmd(SSD1351_CMD_SET_ROW_ADDRESS);
    lcd_data(0);   // top
    lcd_data(127); // bottom
    lcd_cmd(SSD1351_CMD_WRITE_RAM);
    lcd_data_len((U8 *)framebuffer, 128 * 128 * 2);
}

void fb_draw_rect(int x, int y, int end_x, int end_y, U16 clr)
{
    int i, j;
    for (i = y; i <= end_y; i++) {
        for (j = x; j <= end_x; j++) {
            framebuffer[i][j] = (clr >> 8) | (clr << 8);
        }
    }
}

uint8_t fb_draw_bmp(uint8_t left, uint8_t top, uint8_t right, uint8_t bottom, uint8_t *bmp)
{
    U32 datasize;
    U16 bpp;
    U32 bmp_w, bmp_h;
    U8 *bmpdata;
    U32 filesize;
    U32 offset;
    U32 bisize;
    int x;
    int dst_y, dst_w, dst_h;
    // parse header
    if ((bmp[0] != 0x42) || (bmp[1] != 0x4d)) // "BM"
    {
        debugprintf("Invalid BMP-ID\r\n");
        return 1;
    }
    filesize = EXTRACT_READ32(bmp, 2);
    offset   = EXTRACT_READ32(bmp, 10);
    bisize   = EXTRACT_READ32(bmp, 14);
    datasize = filesize - offset;
    bmpdata  = (U8 *)(offset + (U32)(bmp));
    bpp      = (U32)bmp[0x1c] + (U32)(bmp[0x1d] << 8);
    bmp_w    = EXTRACT_READ32(bmp, 0x12);
    bmp_h    = EXTRACT_READ32(bmp, 0x16);

    dst_y = top;
    dst_w = right - left + 1;
    dst_h = bottom - top + 1;
    if (bmp_w < dst_w)
        dst_w = bmp_w;
    if (bmp_h < dst_h)
        dst_h = bmp_h;

    if ((bpp == 8) || (bpp == 8)) {
        return 1;
    } else if (bpp == 24) {
        // convert RGB565
        int ibpl;
        U16 rgb565;
        ibpl = ((bmp_w * 3) + 3) & 0xfffffffc;
        // flip
        bmpdata += ((bmp_h - 1) * ibpl);
        for (dst_y = top; dst_y < (bottom + 1); dst_y++) {
            for (x = 0; x < dst_h; x++) {
                rgb565 = MAKE_RGB565(bmpdata[x * 3 + 2], bmpdata[x * 3 + 1], bmpdata[x * 3 + 0]);
                fb_draw_pixel(left + x, dst_y, rgb565);
            }
            bmpdata -= ibpl; // 4byte align
        }
    }
    return 0; // sucess
}

uint8_t fb_draw_suf(uint8_t left, uint8_t top, uint8_t right, uint8_t bottom, uint8_t *suf)
{
    SURF_HEADER *surfheader;
    int x;
    int dst_y, dst_w, dst_h;

    surfheader = (SURF_HEADER *)suf;
    if (surfheader->bpp != 16)
        return 1;
    if (surfheader->bCompress)
        return 1;

    dst_w = right - left + 1;
    dst_h = bottom - top + 1;

    if (surfheader->w < dst_w)
        dst_w = surfheader->w;

    if (surfheader->h < dst_h)
        dst_h = surfheader->h;

    U16 *buf = (U16 *)(suf + sizeof(SURF_HEADER));
    for (dst_y = top; dst_y < (bottom + 1); dst_y++) {
        for (x = 0; x < dst_w; x++) {
            fb_draw_pixel(left + x, dst_y, buf[x]);
        }
        buf += (surfheader->lpitch / 2);
    }
    return 0; // sucess
}

uint8_t fb_draw_char(uint8_t x, uint8_t y, uint8_t chr, ssd1351_font_t size, uint16_t color)
{
    uint8_t temp, t, t1;
    uint8_t y0    = y;
    uint8_t csize = (size / 8 + ((size % 8) ? 1 : 0)) * (size / 2); /* get size */

    chr = chr - ' ';            /* get index */
    for (t = 0; t < csize; t++) /* write size */
    {
        if (size == 12) /* if size 12 */
        {
            temp = gsc_ssd1351_ascii_1206[chr][t]; /* get ascii 1206 */
        } else if (size == 16)                     /* if size 16 */
        {
            temp = gsc_ssd1351_ascii_1608[chr][t]; /* get ascii 1608 */
        } else if (size == 24)                     /* if size 24 */
        {
            temp = gsc_ssd1351_ascii_2412[chr][t]; /* get ascii 2412 */
        } else {
            return 1; /* return error */
        }
        for (t1 = 0; t1 < 8; t1++) /* write one line */
        {
            if ((temp & 0x80) != 0) /* if 1 */
            {
                fb_draw_pixel(x, y, color);
            }
            temp <<= 1; /* left shift 1 */
            y++;
            if ((y - y0) == size) /* reset size */
            {
                y = y0; /* set y */
                x++;    /* x++ */

                break; /* break */
            }
        }
    }

    return 0; /* success return 0 */
}
uint8_t fb_draw_string(uint8_t x, uint8_t y, char *str, uint32_t color, ssd1351_font_t font)
{
    uint16_t len = strlen(str);
    while ((len != 0) && (*str <= '~') && (*str >= ' ')) /* write all string */
    {
        if (x > (127 - (font / 2))) /* check x point */
        {
            x = 0;              /* set x */
            y += (uint8_t)font; /* set next row */
        }
        if (y > (127 - font)) /* check y pont */
        {
            y = x = 0; /* reset to 0 */
        }
        if (fb_draw_char(x, y, *str, font, color) != 0) /* show a char */
        {
            return 1; /* return error */
        }
        x += (uint8_t)(font / 2); /* x + font/2 */
        str++;                    /* str address++ */
        len--;                    /* str length-- */
    }

    return 0; /* success return 0 */
}
