
#include "demo.h"
#include "etc_driver/lis2dh12_reg.h"

/*
 * @brief  Write generic device register (platform dependent)
 *
 * @param  handle    customizable argument. In this examples is used in
 *                   order to select the correct sensor bus handler.
 * @param  reg       register to write
 * @param  bufp      pointer to data to write in register reg
 * @param  len       number of consecutive register to write
 *
 */

// SDO/SA0 pin pulled high,
// ensuring the on-chip bus address register
// (default setting 0001100x) is set to 00011001.
#define CHIP_BUS_ID (0x19 << 1)
int32_t platform_write(void *handle, uint8_t reg, const uint8_t *bufp,
                       uint16_t len)
{
    uint8_t regs[1];
    regs[0] = reg;
    int i;
    for (i = 0; i < len; i++) {
        twi_write_multi(0, CHIP_BUS_ID, regs, 1, (U8 *)bufp, 1);
        regs[0]++;
        bufp++;
    }
    return 0;
}

/*
 * @brief  Read generic device register (platform dependent)
 *
 * @param  handle    customizable argument. In this examples is used in
 *                   order to select the correct sensor bus handler.
 * @param  reg       register to read
 * @param  bufp      pointer to buffer that store the data read
 * @param  len       number of consecutive register to read
 *
 */
int32_t platform_read(void *handle, uint8_t reg, uint8_t *bufp,
                      uint16_t len)
{
    uint8_t regs[1];
    regs[0] = reg;
    int i;
    for (i = 0; i < len; i++) {
        twi_read(0, CHIP_BUS_ID, regs, 1, bufp, 1);
        regs[0]++;
        bufp++;
    }
    return 0;
}

/*
 * @brief  Send buffer to console (platform dependent)
 *
 * @param  tx_buffer     buffer to transmit
 * @param  len           number of byte to send
 *
 */
void tx_com(uint8_t *tx_buffer, uint16_t len)
{

    uart_putdata(DEBUG_CHANNEL, tx_buffer, len);
}

/*
 * @brief  platform specific delay (platform dependent)
 *
 * @param  ms        delay in ms
 *
 */
void platform_delay(uint32_t ms)
{
    delayms(ms);
}

/*
 * @brief  platform specific initialization (platform dependent)
 */
void platform_init(void)
{
    twi_set_freq(0, 400000);
}

/* Private macro -------------------------------------------------------------*/
#define BOOT_TIME 5 // ms

/* Private variables ---------------------------------------------------------*/

/* Main Example --------------------------------------------------------------*/
void lis2dh12_read_data_polling(void)
{
    /* Initialize mems driver interface */
    stmdev_ctx_t dev_ctx;
    dev_ctx.write_reg = platform_write;
    dev_ctx.read_reg  = platform_read;
    dev_ctx.handle    = 0;
    static int16_t data_raw_acceleration[3];
    static int16_t data_raw_temperature;
    static float acceleration_mg[3];
    static float temperature_degC;
    static uint8_t whoamI;
    static uint8_t tx_buffer1[512];
    static uint8_t tx_buffer2[64];
    static bool binit = false;

    /* Wait boot time and initialize platform specific hardware */
    if (!binit) {
        platform_init();
        /* Check device ID */
        lis2dh12_device_id_get(&dev_ctx, &whoamI);

        if (whoamI != LIS2DH12_ID) {
            debugstring("Device(LIS2DH12) Not Founded\r\n");
            return;
        }
        /* Enable Block Data Update. */
        lis2dh12_block_data_update_set(&dev_ctx, PROPERTY_ENABLE);
        /* Set Output Data Rate to 1Hz. */
        lis2dh12_data_rate_set(&dev_ctx, LIS2DH12_ODR_10Hz);
        /* Set full scale to 2g. */
        lis2dh12_full_scale_set(&dev_ctx, LIS2DH12_2g);
        /* Enable temperature sensor. */
        lis2dh12_temperature_meas_set(&dev_ctx, LIS2DH12_TEMP_ENABLE);
        /* Set device in continuous mode with 12 bit resol. */
        lis2dh12_operating_mode_set(&dev_ctx, LIS2DH12_HR_12bit);
        binit = true;
    }

    /* Read samples in polling mode (no int) */
    while (1) {
        JOYSTICK_TYPE type = get_joystick_type();
        if (type != JOYSTICK_TYPE_NONE)
            return;

        lis2dh12_reg_t reg;
        /* Read output only if new value available */
        lis2dh12_xl_data_ready_get(&dev_ctx, &reg.byte);

        if (reg.byte) {
            /* Read accelerometer data */
            memset(data_raw_acceleration, 0x00, 3 * sizeof(int16_t));
            lis2dh12_acceleration_raw_get(&dev_ctx, data_raw_acceleration);
            acceleration_mg[0] =
                lis2dh12_from_fs2_hr_to_mg(data_raw_acceleration[0]);
            acceleration_mg[1] =
                lis2dh12_from_fs2_hr_to_mg(data_raw_acceleration[1]);
            acceleration_mg[2] =
                lis2dh12_from_fs2_hr_to_mg(data_raw_acceleration[2]);
            sprintf((char *)tx_buffer1,
                    "%4.1f %4.1f %4.1f",
                    acceleration_mg[0], acceleration_mg[1], acceleration_mg[2]);
        }

        lis2dh12_temp_data_ready_get(&dev_ctx, &reg.byte);

        if (reg.byte) {
            /* Read temperature data */
            memset(&data_raw_temperature, 0x00, sizeof(int16_t));
            lis2dh12_temperature_raw_get(&dev_ctx, &data_raw_temperature);
            temperature_degC =
                lis2dh12_from_lsb_hr_to_celsius(data_raw_temperature);
            sprintf((char *)tx_buffer2,
                    "  %6.2f",
                    temperature_degC);
        }
        fb_draw_rect(0, 0, 127, 127, MAKE_RGB565(255, 255, 255));
        fb_draw_string(5, 10, "Acceleration [mg]:", MAKE_RGB565(0, 0, 0), SSD1351_FONT_12);
        fb_draw_string(5, 30, tx_buffer1, MAKE_RGB565(0, 0, 0), SSD1351_FONT_12);
        fb_draw_string(5, 60, "Temperature [degC]:", MAKE_RGB565(0, 0, 0), SSD1351_FONT_12);
        fb_draw_string(5, 80, tx_buffer2, MAKE_RGB565(0, 0, 0), SSD1351_FONT_12);
        fb_update_lcd();
    }
}
