/*****************************************************************************
 *
 * Copyright (C) 2023      Advanced Digital Chips, Inc. All Rights Reserved.
 *						http://www.adc.co.kr
 *
 * THIS SOFTWARE IS PROVIDED BY ADCHIPS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ADCHIPS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 *****************************************************************************/
#include "sdk.h"

static U32 get_timer_source_clock(int nCh)
{
    bool bAsync = false;
    U32 clk = 0;
    if(*(U32 *)TMCLK_ADDR(nCh) & 1)
        bAsync = true;

    const DEV_CLOCK_TYPE clocktype[MAX_TIMER_CHANNEL + 1] = {
        DEV_CLOCK_TIMER_0_1, DEV_CLOCK_TIMER_0_1, DEV_CLOCK_TIMER_2_3, DEV_CLOCK_TIMER_2_3, DEV_CLOCK_TIMER_4};

    if(bAsync)
        clk = get_clock(clocktype[nCh]);
    else
        clk = get_apb_clock();
    return clk;
}
BOOL set_timer(int nCh, U32 ms)
{
    const int timerintnum[] = {INTNUM_TIMER0, INTNUM_TIMER1, INTNUM_TIMER2, INTNUM_TIMER3, INTNUM_TIMER4};
    if(nCh > MAX_TIMER_CHANNEL)
        return FALSE;
    U32 r_tpcon = TPCTRL_ADDR(nCh);
    U32 r_tmcon = TMCTRL_ADDR(nCh);
    U32 r_tmcnt = TMCNT_ADDR(nCh);
    int conval;
    int i   = 0;
    U32 clk = 0;

    U64 nanosecpertick;
    U64 cnt;
    U32 pres;

    U32 scaler[] = {2, 8, 32, 128, 512, 2048, 8192, 32768, 0};

    U32 usec = ms * 1000;

    bool bAsync = false;
    if(*(U32 *)TMCLK_ADDR(nCh) & 1)
        bAsync = true;

    clk = get_timer_source_clock(nCh);

    while(1)
    {
        pres = scaler[i];
        if(pres == 0)
            return FALSE;
        nanosecpertick = 1000000000 / (clk / pres);
        cnt            = ((U64)usec * 1000) / nanosecpertick;

        if(cnt < 0xffffffffLL) // ms is too big to set, adjust PRESACLE.
            break;
        i++;
    }
    // reset

    TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 1 << 1);
    TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 0);

    conval = i << 1;
    TIMER_REG_WRITE(bAsync, nCh, r_tmcnt, cnt);
    TIMER_REG_WRITE(bAsync, nCh, r_tmcon, conval | 1);
    //*(vU32*)r_tmcon = conval;
    enable_interrupt(timerintnum[nCh], TRUE);
    return TRUE;
}

BOOL stop_timer(int nCh)
{
    if(nCh > MAX_TIMER_CHANNEL)
        return FALSE;

    bool bAsync = false;
    if(*(U32 *)TMCLK_ADDR(nCh) & 1)
        bAsync = true;
    U32 r_tmcon = TMCTRL_ADDR(nCh);
    TIMER_REG_WRITE(bAsync, nCh, r_tmcon, 0); // stop
    return TRUE;
}

/*
"period_clock_cnt" must be greater than 3.
"duty_clock_cnt" must be greater than 2.
if async mode and Xin is 12Mhz, Timer Input Clock is 12Mhz/2(prescale)=6Mhz
Therefore, the maximum frequency is 6Mhz/3, which is 2Mhz.
     
            _____________
  _________|            |
  |<= period clock cnt ->|
  |<- duty->|          
*/

int set_pwm_ex(int nCh,int period_clock_cnt,int duty_clock_cnt,bool start_level,int interrupt_pluse_number,bool one_time)
{
    if(period_clock_cnt<3)
        period_clock_cnt = 3;
    
    if(duty_clock_cnt<2)
        duty_clock_cnt = 2;

    U32 r_tpcon = TPCTRL_ADDR(nCh);
    U32 r_tmcon = TMCTRL_ADDR(nCh);
    U32 r_tmcnt = TMCNT_ADDR(nCh);
    U32 r_tmdut = TMDUT_ADDR(nCh);
    U32 r_tmpul = TMPUL_ADDR(nCh);
    bool bAsync = false;

    if(*(U32 *)TMCLK_ADDR(nCh) & 1)
        bAsync = true;

    U32 clk=get_timer_source_clock(nCh);
    clk/=2;// prescale is 2
    U32 actually_clk = clk/period_clock_cnt;
    
    TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 1 << 1);
    TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 0);

    //int prescale   = 2;
    int f_prescale = 0 << 1;
    // if f_prescale < 3, see datasheet 
    duty_clock_cnt-=1;
    period_clock_cnt-=1;

    TIMER_REG_WRITE(bAsync, nCh, r_tmcnt, period_clock_cnt);
    TIMER_REG_WRITE(bAsync, nCh, r_tmdut, duty_clock_cnt);
    TIMER_REG_WRITE(bAsync, nCh, r_tmpul, interrupt_pluse_number);

    int conval = F_TMCTRL_TMOD_PWM  | f_prescale|F_TMCTRL_TMEN;
    if(one_time)
        conval |= F_TMCTRL_PWMO ;
    if(start_level)
        conval |= F_TMCTRL_PWML_HIGH;

    TIMER_REG_WRITE(bAsync, nCh, r_tmcon, conval);
    return actually_clk;
}

/**
 * @brief Set the pwm object
 * 
 * @param nCh channel
 * @param hz frequency
 * @param interrupt_pulse_number interrupt pulse number
 * @param bperiod false is one-time generation
 * @param en enable or disable
 * @return actually frequency
 *  if Async mode and Xin is 12Mhz, Timer Input Clock is 6Mhz (12Mhz/2(prescale))
 *  max freq : 6/3=2Mhz
 *  6/4 =1.5Mhz
 *  6/5 =1.2Mhz
 *  6/6 =1Mhz....
 */
int set_pwm(int nCh, int hz, int interrupt_pulse_number, bool bperiod, bool en)
{
    if(nCh > MAX_TIMER_CHANNEL)
        return 0;

    bool bAsync = false;

    if(*(U32 *)TMCLK_ADDR(nCh) & 1)
        bAsync = true;

    U32 r_tmcon = TMCTRL_ADDR(nCh);
    if(en==false)
    {
        TIMER_REG_WRITE(bAsync, nCh, r_tmcon, 0);
        return 0;
    }

    U32 clk = 0;
    clk = get_timer_source_clock(nCh);
    clk/=2;// prescale is 2
    int period = clk/ hz;
    int duty   = period / 2;
    bool ontime = !bperiod;
    
    return set_pwm_ex(nCh,period,duty,0,interrupt_pulse_number,ontime);
}
