
#include "sdk.h"
#include "fatfs/diskio.h"


typedef struct _tagDeviceInfo
{
    U32 fat_offset_sector;
    U32 sector_size;
    U32 fat_sector_count;
	U32 flash_sector_cnt[2];
} DEVICE_INFO;

DEVICE_INFO myflash;
/**
 * @brief return Flash Number and Set sector number of Target Flash.
 * 
 * @param sector logical sector number
 * @param pPhy_sector_of_dev pointer to save physical number of flash
 * @return Flash index number 
 */
inline static int get_dev(LBA_t sector, U32* pPhy_sector_of_dev)
{
	U32 phy_number = sector+myflash.fat_offset_sector;
	if(myflash.flash_sector_cnt[0] > phy_number)
	{
		*pPhy_sector_of_dev = phy_number;
		return 0;
	}
	else 
	{
		*pPhy_sector_of_dev = phy_number - myflash.flash_sector_cnt[0];
		return 1;
	}
}

U32 flash_fs_sector_size()
{
	return myflash.sector_size;
}

LBA_t flash_fs_sector_count()
{
	return (LBA_t)myflash.fat_sector_count;
}

DRESULT flash_fs_flushdata()
{	
	//do nothing
	return RES_OK;
}

static BOOL flash_fs_updatesect( LBA_t sector, void* buffer )
{	
	U32 phy_sector;
	U32 flash_num = get_dev(sector,&phy_sector);
	U32 offset = phy_sector * myflash.sector_size;
	sflash_erase_sector(flash_num,phy_sector,1);
	sflash_dev_write(flash_num,offset,buffer,myflash.sector_size);
	return TRUE;
}
/**
 * @brief Writes data to the flash.
 * 
 * @param startsector sector number to write
 * @param buf A pointer to the buffer containing the data to be written
 * @param cnt The number of sectors to be written
 * @return RES_OK if it succeeds, RES_ERROR if it fails.
 */
DRESULT flash_fs_updatesects( LBA_t startsector, const BYTE* buf, UINT cnt )
{		
	int i;
	//debugprintf("%s(%d,%d,%d)\r\n",__FUNCTION__,startsector,buf,cnt);
	for( i = 0; i < cnt; i++ )
	{		
		if( flash_fs_updatesect( startsector + i, ( void* )( ( ( U32 )buf ) + ( i * myflash.sector_size) ) ) == FALSE )
			return RES_ERROR;
	}	
	return RES_OK;
}

static BOOL flash_fs_readsect( U32 sector, void* buffer )
{
	U32 phy_sector;
	U32 flash_num = get_dev(sector,&phy_sector);
	U32 offset = phy_sector * myflash.sector_size;
	sflash_dev_read(flash_num,offset, buffer, myflash.sector_size);
	return TRUE;
}

/**
 * @brief Reads data from the flash.
 * @param startsector sector number to read.
 * @param buf A pointer to the buffer that receives the data read.
 * @param cnt The number of sectors to read.
 * @return RES_OK if it succeeds, RES_ERROR if it fails.
 */
DRESULT flash_fs_readsects(LBA_t startsector,BYTE* buf,UINT cnt)
{				
	int i = 0;	
	U8* u8buf = (U8*)buf;
	//debugprintf("%s(%d,%d,%d)\r\n",__FUNCTION__,startsector,buf,cnt);
	for(i=0;i<cnt;i++)
	{
		if( flash_fs_readsect( startsector + i, u8buf + ( i * myflash.sector_size) ) == FALSE )
			return RES_ERROR;		
	}		
	return RES_OK;	
}
/**
 * @brief Initialize the parameters.
 * @return RES_OK if it succeeds, RES_ERROR if it fails.
 */
DRESULT flash_fs_init()
{
    myflash.fat_offset_sector=FS_START_SECTOR;
	myflash.sector_size = sflash_get_sector_size(0);
	if(myflash.sector_size==0)
		return 	RES_ERROR;
    myflash.flash_sector_cnt[0] = (U32)sflash_get_sector_count(0);
    myflash.flash_sector_cnt[1] = (U32)sflash_get_sector_count(1);
    myflash.fat_sector_count = myflash.flash_sector_cnt[0] + myflash.flash_sector_cnt[1] - myflash.fat_offset_sector;
	return RES_OK;
}

static BOOL my_init()
{
	if(flash_fs_init()==RES_OK)
		return TRUE;
	return FALSE;
}
static BOOL my_sync()
{
	if(flash_fs_flushdata()==RES_OK)
		return TRUE;
	return TRUE;
}

static BOOL my_readblock(int blocknum, void* buf, int blockcnt)
{
	if(flash_fs_readsects((LBA_t)blocknum,(BYTE*)buf,(UINT)blockcnt)==FR_OK)
		return TRUE;
	return FALSE;
}
static BOOL my_writeblock(int blocknum, void* buf, int blockcnt)
{
	if(flash_fs_updatesects((LBA_t)blocknum,(BYTE*)buf,(UINT)blockcnt)==FR_OK)
		return TRUE;
	return FALSE;
}

static U32 my_block_size()
{
	return flash_fs_sector_size();
}

static U32 my_block_count()
{
	return (U32)flash_fs_sector_count();
}

static BLOCKDEV flash_dev;
/**
 * @brief Initialize BLOCKDEV 
 * @return 
 */
BLOCKDEV* flash_fs_getdev()
{
	memset(&flash_dev,0,sizeof(BLOCKDEV));
	memcpy(flash_dev.name,"flash",5); //device name
	flash_dev.init = my_init;
	flash_dev.readblock = my_readblock;
	flash_dev.writeblock = my_writeblock;
	flash_dev.get_block_count = my_block_count;
	flash_dev.get_block_size = my_block_size;
	flash_dev.sync = my_sync;
	return &flash_dev;
}

